// rf69 demo tx rx.pde
// -*- mode: C++ -*-
// Example sketch showing how to create a simple messageing client
// with the RH_RF69 class. RH_RF69 class does not provide for addressing or
// reliability, so you should only use RH_RF69  if you do not need the higher
// level messaging abilities.
// It is designed to work with the other example rf69_server.
// Demonstrates the use of AES encryption, setting the frequency and modem
// configuration

// **********************************************************************************
// Copyright Felix Rusu 2018, http://www.LowPowerLab.com/contact
// **********************************************************************************
// License
// **********************************************************************************
// This program is free software; you can redistribute it
// and/or modify it under the terms of the GNU General
// Public License as published by the Free Software
// Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will
// be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
//
// Licence can be viewed at
// http://www.gnu.org/licenses/gpl-3.0.txt
//
// Please maintain this license information along with authorship
// and copyright notices in any redistribution of this code

#include <LowPower.h> //get library from: https://github.com/lowpowerlab/lowpower
//writeup here: http://www.rocketscream.com/blog/2011/07/04/lightweight-low-power-arduino-library/

#include <SPI.h>
#include <RH_RF69.h>
#include <TimeLib.h>
/************ Radio Setup ***************/

// Change to 434.0 or other frequency, must match RX's freq!
#define RF69_FREQ  915
#define RFM69_INT    2
#define RFM69_CS    10
#define RFM69_RST   24
int hours = 18;
int minutes = 56;
int seconds = 0;
int days = 1;
int months = 2;
int years = 2020;
// Singleton instance of the radio driver
RH_RF69 rf69(RFM69_CS, RFM69_INT);

int16_t packetnum = 0;  // packet counter, we increment per xmission

// state, mode, ledpins initialization
bool awake = true;
int mode = 1;
const int NUM_LEDS = 4;
int selected[NUM_LEDS];
int ledPins[] =  {3, 5, 6, 9};
//int ledPins[] =  {3, 4, 23, 5, 6, 7, 8, 9};

// timers and animation
#include <elapsedMillis.h>
elapsedMillis animationElapsed;
int animationValue = 0;
int animationDuration = 500;
int brightness = 255;

void setup()
{
  Serial.begin(9600);
  //  while (!Serial) { delay(1); } // wait until serial console is open, remove if not tethered to computer
  for (int i = 0; i < NUM_LEDS; i++) {
    pinMode(ledPins[i], OUTPUT);
    selected[i] = 0;
  }
  selected[0] = 1;
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, LOW);

  Serial.println("Feather RFM69 RX Test!");
  Serial.println();

  // manual reset
  digitalWrite(RFM69_RST, HIGH);
  delay(10);
  digitalWrite(RFM69_RST, LOW);
  delay(10);

  if (!rf69.init()) {
    Serial.println("RFM69 radio init failed");
    digitalWrite(23, HIGH);
    digitalWrite(6, HIGH);
    digitalWrite(5, HIGH);
    digitalWrite(4, HIGH);s
    digitalWrite(3, HIGH);
    delay(30);
    digitalWrite(3, LOW);
    delay(30);
    while (1);
  }
  Serial.println("RFM69 radio init OK!");

  digitalWrite(23, HIGH);
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  delay(50);
  digitalWrite(4, LOW);
  delay(50);
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM (for low power module)
  // No encryption
  if (!rf69.setFrequency(RF69_FREQ)) {
    Serial.println("setFrequency failed");
  }

  // If you are using a high power RF69 eg RFM69HW, you *must* set a Tx power with the
  // ishighpowermodule flag set like this:
  rf69.setTxPower(20, true);  // range from 14-20 for power, 2nd arg must be true for 69HCW

  // The encryption key has to be the same as the one in the server
  uint8_t key[] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08
                  };
  rf69.setEncryptionKey(key);

  Serial.print("RFM69 radio @");  Serial.print((int)RF69_FREQ);  Serial.println(" MHz");
  setAllOff();
  //start_0();
}

void loop() {
  //receive();
  if (awake == true) {
    
    if (animationElapsed > 2*animationDuration) {
//      start_0();
      start_chase_0();
      animationElapsed = 0;
    }
    else if (animationElapsed > animationDuration) {
      animationValue = 2*animationDuration - animationElapsed;
    }
    else {
      animationValue = animationElapsed;
    }
    //    calc_percent_remaining();
    
    if (mode == 0) {
      animate_digital();
    }
    if (mode == 1) {
      animate_fade();
    }
  }

  if (awake == false) {
    //LowPower.powerSave(SLEEP_15MS, ADC_ON, BOD_OFF, TIMER2_ON);
    LowPower.powerDown(SLEEP_8S, ADC_ON, BOD_OFF);
  }
  delay(5);
}

void Blink(byte PIN, byte DELAY_MS, byte loops) {
  for (byte i = 0; i < loops; i++)  {
    digitalWrite(PIN, HIGH);
    delay(DELAY_MS);
    digitalWrite(PIN, LOW);
    delay(DELAY_MS);
  }
}
void digitalClockDisplay() {
  // digital clock display of the time
  Serial.println(" ");
  Serial.print("The time is now: ");
  Serial.print(hour());
  Serial.print(":");
  Serial.print(minute());
  Serial.print(":");
  Serial.print(second());
  Serial.println(" ");
}

void start_0() {
  for (int i = 0; i < NUM_LEDS; i++) {
    if (i % 2 == 0) {
      selected[i] = 1;
    } else {
      selected[i] = 0;
    }
  }
}

void start_chase_0() {
  // at every loop, flip the value of the next index to one
  // and self to zero
  for (int i = NUM_LEDS - 1; i >= 0 ; i--) {
    // wrap around condition, last led to 0, first led to 1
    if (selected[NUM_LEDS - 1] == 1){
      selected[NUM_LEDS - 1] = 0;
      selected[0] = 1;
      break;
    }
    else if (selected[i] == 1) {
      selected[i] = 0;
      selected[i + 1] = 1;
      break;
    }
  }
}

void animate_fade() {
  float percentComplete = (((float)animationValue) / (float)animationDuration);
  // check each led
  for (int i = 0; i < NUM_LEDS; i++) {
    float fade = percentComplete; // fade up
    if (selected[i] == 0) {
      fade = 0.0;
    }
    // set the one on chase to fade up
    setBrightness(i, fade * brightness); //*random(1,3)); <- gives a sparkle
  }
}

void animate_digital() {
  for (int i = 0; i < NUM_LEDS; i++) {
    setLed(i, selected[i] == 1);
  }
}

void receive() {
  if (rf69.available()) {
    // Should be a message for us now
    uint8_t buf[RH_RF69_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);
    if (rf69.recv(buf, &len)) {
      if (!len) return;
      receive_debug(len, buf);
      buf[len] = 0;
      if (strstr((char *)buf, "Hello World")) {
        // Send a reply!
        send_ack();
      } else {
        receive_fail();
      }
    }
  }
}

void receive_fail() {
  Serial.println("Receive failed");
  digitalWrite(23, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);

  digitalWrite(6, HIGH);
  delay(30);
  digitalWrite(6, LOW);
  delay(30);
}
void send_ack() {
  uint8_t data[] = "And hello back to you";
  rf69.send(data, sizeof(data));
  rf69.waitPacketSent();
  Serial.println("Sent a reply");
  digitalWrite(23, HIGH);
  digitalWrite(6, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  // reply leds
  digitalWrite(5, HIGH);
  delay(50);
  digitalWrite(5, LOW);
  delay(50);
  digitalWrite(5, HIGH);
  delay(50);
  digitalWrite(5, LOW);
  delay(50);
}

void receive_debug(uint8_t len, uint8_t *buf) {
  Serial.print("Received [");
  Serial.print(len);
  Serial.print("]: ");
  Serial.println((char*)buf);
  Serial.print("RSSI: ");
  Serial.println(rf69.lastRssi(), DEC);
  digitalWrite(6, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(3, HIGH);
  digitalWrite(4, HIGH);
  receive_leds_on();
}

void receive_leds_on() {
  digitalWrite(23, HIGH);
  delay(100);
  digitalWrite(23, LOW);
  delay(100);
}

void setLed(int i, bool isOn) {
  digitalWrite(ledPins[i], !isOn);
}

// fade is flaot scaled between 0.0 and brightness
void setBrightness(int i, float fade) {
  analogWrite(ledPins[i], (int)(brightness - fade));
}

void setAllOff() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, false);
}

void setAllOn() {
  for (int i = 0; i < NUM_LEDS; i++)
    setLed(i, true);
}
